---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
--	A simple dialog to offer a quest to go 'kill party X'.
--	The core components of the quest are:
--		1) The hook, what is said or given to offer the quest?
--		2) Accept, what happens when the party accepts the quest
--		3) Decline, what happens when the party declines the request
--		4) Remind, what happens when the party has already accepted
--			or declined, and talks to the quest giver again
--		5) Success, what happens when the killing is done
--		6) Unsolicited succes, what happens when a party is
--			already dead before the quest is offered
--		7) The target, what are we supposed to kill?!?
--
--	Management of state and persistence is handled by a plain data
--	table called 'kill_quest'. The quest table holds all relevant 
--	quest state data, and can be easily archived and unarchived.
--
Quests = {}
Quests.kill_quest = 
{
	ident = "kill_quest",

	hook = [[HOOK]],
	
	accept = [[ACCEPT]],
	
	accept_handler = function (self, me, you)
		self.accepted = true
		MiniMap.insert_marker(...)
		-- Some mechanism to remove the marker.
	end,
	
	decline = [[DECLINE]],
	
	decline_handler = function (self, me, you)
		if me.party:affinity(you.party) <= 0 then
			Env.begin_combat(...)
		end
	end,
	
	remind = [[REMIND]],
	
	success = [[SUCCESS]],
	
	success_handler = function (self, me, you)
		if self.betray then
			-- Combat bizatch!
		elseif self.reward_item then
			for i=1,#me.inventory do
				local item = me.inventory[i].name
				if item == self.reward_item then
					me:remove_item(item)
					if ContainerSpace ... then
						you:insert_item(item) 
					else
						-- Oh noes, you're full!
					end
					break
				end
			end
		end
		
		if self.ally > 0 then
			me.party.awareness[you.party] = self.ally
		end
	end,
	
	unsolicited_success = [[UNSOLICITED]],
	
	unsolicited_success_handler = function (self, me, you)
		if self.ally > 0 then
			me.party.awareness[you.party] = self.ally
		end
	end,
	
	-- What is the target party!
	target = nil,
}

local function archive_quest (quest)
	-- TODO: Generalize
	local ret = {}
	for k,v in pairs(quest) do
		if type(v) ~= "function" then
			ret[k] = v
		end
	end
	return ret
end

local function unarchive_quest (quest)
	assert(quest.ident)
	local ret = Copy(Quests[quest.ident])
	CopyTable(quest, ret)	
	return ret
end

Archiver:set_custom  ("quest", archive_quest)
Unarchiver:set_custom("quest", unarchive_quest)
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
local kill_quest =
{
	talk_entry = function (self, me, you)		
		-- Return values.
		local text, options
		local target = NPC:get(me.quest.target)
		
		-- Target party is dead.
		if not target or #target.members == 0 then
			-- Unsolicited
			if not me.quest.accepted then
				text = me.quest.unsolicited_success
				me.quest:unsolicited_success_handler(me, you)
				
			-- Finished quest.
			else
				text = me.quest.success
				me.quest:success_handler(me, you)
			end
			
			-- The quest is forever done.
			me.quest = nil
			
		-- Target is still alive.
		else
			if not me.quest.offered then
				text = me.quest.hook
				options = {self.accept, self.decline}
				me.quest.offered = true
				
			elseif me.quest.accepted then
				text = me.quest.remind
			else
				text = me.quest.refuse
			end
		end
	
		return {text, options}
	end,

	escape = function () end,
	
	accept = function (self, me, you)
		me.quest:accept_handler(me, you)
		return {me.quest.accept, nil}
	end,
	
	decline = function (self, me, you)
		me.quest:decline_handler(me, you)
		return {me.quest.decline, nil}
	end
	
	ident = "kill_quest",
}

local function KillQuestTemplate (
	hook,
	accept,
	refuse,
	remind,
	success,
	reward,
	betray)
	
	local quest = Copy(kill_quest_data)
	assert(hook and accept and refuse and remind and success)
	
	quest.hook 		= hook
	quest.accept 	= accept
	quest.refuse 	= refuse
	quest.remind 	= remind
	quest.success 	= success
	quest.reward 	= reward
	quest.betray	= false
	
	return quest
end
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------
-- Return some kill quest templates.
return 
{
	KillQuestTemplate(
		[[Oh my poor puppy! He was stomped on by a cruel and viscious @RACE@ from the @TRIBE@ tribe! I'm so fraught with grief, please exact vengeance on the monster who took away my jiggly puppy!]],
		
		[[Oh thank you kind sir! I can't sleep at night knowing that there is puppy killer at large. I will show my gratitude in whatever humble way that I can.]],
		
		[[Oh poo! You're an antipuppyist! Leave me alone you monster!]],
		
		[[Please put an end to the @TRIBE@ tribe's evil puppy killing ways!]],
		
		[[Oh thank you! I feel so much better knowing that there is one less puppy killing monster lurking in the forest. Please accept a token of my gratitude. Now if you'll exuse me, I've got to go check on my kitten stew...]],
		
		-- Should give reward.
		true,
	),
	
	KillQuestTemplate(
		[[Hark, Adventurers! I offer you truce in exchange for your service, kill my sworn enemy and we shall have peace.]],
		
		[[Very well, you may have safe passage through my land, but make haste in killing the @RACE@s of @TRIBE@.]],
		
		[[You have made a foolish decision! Taste my wrath!]],
		
		[[Kill my foe and we shall have peace.]],
		
		[[Foolish]],
		
		-- Should give reward.
		false,
		
		-- Betrayal?
		true,
	),
}